#!/bin/ksh

[ $AAPP_PREFIX ] || { echo "AAPP_PREFIX is not defined"; exit; }
[ -d "$AAPP_PREFIX" ] || { echo "AAPP_PREFIX is not a directory"; exit; }

. $AAPP_PREFIX/ATOVS_CONF

file_identifier=$1     #optional - first part of file name e.g. iasi

export USE_OB_TIME=Y

encode_prog=aapp_encodebufr_1c
decode_prog=aapp_decodebufr_1c
indir=$PWD/infiles
bufr_dir=$PWD/bufrdc_bufr
decoded_dir=$PWD/bufrdc_decoded
mkdir -p $bufr_dir $decoded_dir

#----Pre-process FY-3 if not already done

cd $indir
if [ "$file_identifier" = "" ] || [ "$file_identifier" = "mwhs" ]; then
  for file in $(ls mwhs*.HDF); do
    outfile=${file%.HDF}.l1c
    if [ ! -f $outfile ]; then
      echo "mwhs2_sdr $file"
      mwhs2_sdr $file
    fi
  done
fi
if [ "$file_identifier" = "" ] || [ "$file_identifier" = "iras" ]; then
  for file in $(ls iras*.HDF); do
    outfile=${file%.HDF}.l1c
    if [ ! -f $outfile ]; then
      echo "iras_sdr $file"
      iras_sdr $file
    fi
  done
fi
if [ "$file_identifier" = "" ] || [ "$file_identifier" = "mwri" ]; then
  for file in $(ls mwri*.HDF); do
    outfile=${file%.HDF}.l1c
    if [ ! -f $outfile ]; then
      echo "mwri_sdr $file"
      mwri_sdr $file
    fi
  done
fi

export BUFR_TABLES_SAVE=$BUFR_TABLES

#----Go through the .l?? files, determining the instrument

files=$(cd $indir; ls ${file_identifier}*.l??)
for file in $files; do

  export BUFR_TABLES=$BUFR_TABLES_SAVE
  export MASTER_TABLE=   #use the default MASTER_TABLE unless specified otherwise
  flag=                  #use the defaults. Other possibilities are "-v none", "-v yes", "-N", "-n"

  instr=""
  if [[ $file = amsual1c* ]]; then
    instr=AMSU-A
  elif [[ $file = hirsl1c* ]]; then
    instr=HIRS
  elif [[ $file = mhsl1c* ]]; then
    instr=MHS
  elif [[ $file = hirsl1d* ]]; then
    instr=HIRS1D
    export BUFR_TABLES=$AAPP_PREFIX/AAPP/data/bufrtables/
    export MASTER_TABLE=15
  elif [[ $file = iasi*.lpc ]]; then
    instr=PCIASI
  elif [[ $file = iasi*.l1c ]]; then
    instr=IASI
    export ENHANCED_IASI=Y
  elif [[ $file = SATMS* ]]; then
    instr=ATMS
  elif [[ $file = SCRIS* ]]; then
    instr=CRIS
  elif [[ $file = mwhs* ]]; then
    instr=MWHS2
  elif [[ $file = iras* ]]; then
    instr=IRAS
  elif [[ $file = mwri* ]]; then
    instr=MWRI
  fi

  [ "$BUFR_TABLES" ] || { echo "BUFR_TABLES is not defined"; exit; }
  [ -d "$BUFR_TABLES" ] || { echo "BUFR_TABLES is not a directory"; exit; }

#----BUFR encode

  if [ "$instr" ]; then
    $encode_prog $flag -i $indir/$file $instr 
    if [ $? = 0 ]; then
      bufrfile=${file%.l??}.bufr
      mv $indir/$bufrfile $bufr_dir

#----BUFR decode

      if [[ $instr != *1D && $instr != MWRI ]]; then    #decode not implemented
        export ENHANCED_IASI=
        $decode_prog -i $bufr_dir/$bufrfile $instr 
        if [ $? = 0 ]; then
          mv $bufr_dir/$file $decoded_dir
          echo "created $decoded_dir/$file"
        else
          echo "$decode_prog failed"
          exit 1
        fi
      else
        echo "BUFRDC decode not implemented in AAPP for $instr"
      fi

    else
      bufrfile=${file%.l1*}.bufr
      rm -f $bufrfile
      echo "$encode_prog failed"
      exit 1
    fi
  fi
done

