#!/bin/ksh

[ $AAPP_PREFIX ] || { echo "AAPP_PREFIX is not defined"; exit; }
[ -d "$AAPP_PREFIX" ] || { echo "AAPP_PREFIX is not a directory"; exit; }

. $AAPP_PREFIX/ATOVS_CONF

file_identifier=$1     #optional - first part of file name e.g. iasi, hirs, etc.

if [ "$file_identifier" = "" ] || [[ "$file_identifier" = hirs* ]]; then
  if [ ! "$ECCODES_DEFINITION_PATH" ]; then
    echo 'NOT running the HIRS 1d encoding'
    echo 'To run it, you would need to define environment variable ECCODES_DEFINITION_PATH'
    echo 'e.g. export ECCODES_DEFINITION_PATH=$DIR_ECCODES/share/eccodes/definitions:$AAPP_PREFIX/AAPP/data/tools_eccodes/definitions'
    echo 'where $DIR_ECCODES is the ecCodes installation directory'
  fi
fi

encode_prog=eccodes_encodebufr_1c
decode_prog=eccodes_decodebufr_1c
indir=$PWD/infiles
bufr_dir=$PWD/eccodes_bufr
decoded_dir=$PWD/eccodes_decoded
mkdir -p $bufr_dir $decoded_dir

#----Pre-process FY-3 if needed

cd $indir
if [ "$file_identifier" = "" ] || [ "$file_identifier" = "mwhs" ]; then
  for file in $(ls mwhs*.HDF); do
    outfile=${file%.HDF}.l1c
    if [ ! -f $outfile ]; then
      echo "mwhs2_sdr $file"
      mwhs2_sdr $file
    fi
  done
fi
if [ "$file_identifier" = "" ] || [ "$file_identifier" = "iras" ]; then
  for file in $(ls iras*.HDF); do
    outfile=${file%.HDF}.l1c
    if [ ! -f $outfile ]; then
      echo "iras_sdr $file"
      iras_sdr $file
    fi
  done
fi
if [ "$file_identifier" = "" ] || [ "$file_identifier" = "mwri" ]; then
  for file in $(ls mwri*.HDF); do
    outfile=${file%.HDF}.l1c
    if [ ! -f $outfile ]; then
      echo "mwri_sdr $file"
      mwri_sdr $file
    fi
  done
fi

#----Go through the .l?? files, determining the instrument

files=$(cd $indir; ls ${file_identifier}*.l??)
for file in $files; do

  export MASTER_TABLE=       #use the default MASTER_TABLE unless specified otherwise
  flag=                      #use the defaults. Other possibilities are "-v none", "-v yes", "-N", "-n"

  instr=""
  if [[ $file = amsual1c* ]]; then
    instr=AMSU-A
  elif [[ $file = hirsl1c* ]]; then
    instr=HIRS
  elif [[ $file = mhsl1c* ]]; then
    instr=MHS
  elif [[ $file = hirsl1d* ]] && [ "$ECCODES_DEFINITION_PATH" ]; then
    instr=HIRS1D
    export MASTER_TABLE=15
  elif [[ $file = iasi*.lpc ]]; then
    instr=PCIASI
  elif [[ $file = iasi*.l1c ]]; then
    instr=IASI
  elif [[ $file = SATMS* ]]; then
    instr=ATMS
  elif [[ $file = SCRIS* ]]; then
    instr=CRIS
  elif [[ $file = mwhs* ]]; then
    instr=MWHS2
  elif [[ $file = iras* ]]; then
    instr=IRAS
  elif [[ $file = mwri* ]]; then
    instr=MWRI
  fi

#----BUFR encode

  if [ "$instr" ]; then
    $encode_prog $flag -i $indir/$file $instr 
    if [ $? = 0 ]; then
      bufrfile=${file%.l??}.bufr
      mv $indir/$bufrfile $bufr_dir

#----BUFR decode

      if [[ $instr != *1D ]]; then     #decode not implemented for 1d
        echo "call $decode_prog"
        echo "$decode_prog -i $bufr_dir/$bufrfile $instr"
        $decode_prog -i $bufr_dir/$bufrfile $instr 
        if [ $? = 0 ]; then
          mv $bufr_dir/$file $decoded_dir
          echo "created $decoded_dir/$file"
        else
          echo "$decode_prog failed"
          exit 1
        fi
      fi

    else
      bufrfile=${file%.l1*}.bufr
      rm -f $bufrfile
      echo "$encode_prog failed"
      exit 1
    fi
  fi
done
